unit Marksmain;

interface

uses
  Winapi.Windows, Winapi.Messages, PsAPI, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VclTee.TeeGDIPlus, Vcl.StdCtrls, Math, Types,
  VCLTee.TeeTools, VCLTee.TeEngine, VCLTee.TeeLegendScrollBar, VCLTee.TeeProcs,
  VCLTee.Chart, Vcl.ExtCtrls, VCLTee.Series;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    SpectraChart: TChart;
    LegendScrollbar: TLegendScrollBar;
    NearestTool: TNearestTool;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    ChartTool1: TDragMarksTool;
    procedure Button1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure SpectraChartMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SpectraChartMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SpectraChartMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    procedure UpdateMemoryUse;
    procedure SeriesAfterDrawValues(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Generics.Collections;

type
  TMyMarksItem=class
  private
    ValueIndex: Integer;
    Mark: TMarksItem;
    Position: TSeriesMarkPosition;

    Constructor Create(AOwner:TCustomTeePanel);
  end;

  TMyMarksItems=class(TObjectList<TMyMarksItem>);

  TMySeriesMarks=class(TObjectList<TMyMarksItems>);

  TCustomTextShapeAccess=class(TCustomTextShape);

  TMyDragMarksTool=class(TTeeCustomToolSeries)
  private
    SeriesIndex: Integer;
    ValueIndex: Integer;
    IOldPos: TPoint;

    Constructor Create(AOwner:TComponent); override;
  end;

var seriesMarks: TMySeriesMarks;
    dragMarksTool: TMyDragMarksTool;
    visibleMarks: Integer;

Constructor TMyMarksItem.Create(AOwner:TCustomTeePanel);
begin
  Mark:=TMarksItem.Create(AOwner);
end;

Constructor TMyDragMarksTool.Create(AOwner:TComponent);
begin
  SeriesIndex:=-1;
  ValueIndex:=-1;
  IOldPos:=Point(-1,-1);
end;

function CurrentMemoryUsage: Cardinal;
var
   pmc: TProcessMemoryCounters;
begin
   pmc.cb := SizeOf(pmc) ;
   if GetProcessMemoryInfo(GetCurrentProcess, @pmc, SizeOf(pmc)) then
     Result := pmc.WorkingSetSize
   else
     RaiseLastOSError;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
  i : Integer;
  Series1: TFastLineSeries;
begin
  Series1 := TFastLineSeries.Create(Self);
  Series1.XValues.Order := loNone;
  Series1.DrawAllPoints:=False;
  Series1.DrawAllPointsStyle:=daMinMax;
  for i := 1 to 65000 do
  begin
    Series1.AddXY(i, sin(i/1000+SpectraChart.SeriesCount), '');
  end;
  SpectraChart.AddSeries(Series1);
  SpectraChart.LeftAxis.MaximumOffset:=100;
  SpectraChart.LeftAxis.MinimumOffset:=20;
  Series1.AfterDrawValues:=SeriesAfterDrawValues;
  UpdateMemoryUse;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  i,j:Integer;
begin
  Screen.Cursor:=crHourglass;
  seriesMarks.Clear;
  for j:=0 to SpectraChart.SeriesCount-1 do
  begin
    seriesMarks.Add(TMyMarksItems.Create);
    visibleMarks:=0;
    SpectraChart.Series[j].CalcVisiblePoints:=false;     // otherwise most marks won't be shown
    with SpectraChart.Series[j].Marks do
    begin
      BackColor:=SpectraChart.Color;
      Automatic.Resize:=true;
      Automatic.Wrap:=true;
      Automatic.Rotate:=true;
      Automatic.Hide:=true;
      ArrowLength:=round(SpectraChart.Height/20);
      Arrow.Color:= SpectraChart.Series[j].Color;
      Arrow.Visible:=true;
      Arrow.Width:=2;
      OnTop:=true;
      Clip:=true;
      SoftClip:=false;
      Autoposition:=false;
      Callout.Transparency:=30;
      Callout.Distance:=5;
    end;
    for i:=1 to 65000 do
    begin
      if (CompareValue(sin(i/1000+j), 1, 0.000005)=EqualsValue) then // show marks at maxima as "peak label"
      begin
        seriesMarks[j].Add(TMyMarksItem.Create(SpectraChart));
        with seriesMarks[j].Last, Mark do
        begin
          ValueIndex:=i;
          Text.Text:=IntToStr(i);
          Transparent:=false;
          Frame.Color:=SpectraChart.Color;
          Font.Color:=clBlack;
          Angle:=90;
          Frame.Color:=clnone;
          Shadow.Visible:=false;
          ClipText:=false;
        end;
        Inc(visibleMarks);
      end;
    end;
  end;
  Screen.Cursor:=crDefault;
  UpdateMemoryUse;
end;

procedure TForm1.Button3Click(Sender: TObject);
var
  i: Integer;
begin
  for i:=SpectraChart.SeriesCount-1 downto 0 do
  begin
    Spectrachart.Series[i].Free;
  end;
  UpdateMemoryUse;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  seriesMarks:=TMySeriesMarks.Create;
  dragMarksTool:=TMyDragMarksTool.Create(SpectraChart);

    with SpectraChart do
    begin
      Tabstop:=true;
      ClipPoints:=True;
      Gradient.Visible:=False;
      Color:=clWhite;
      Walls.Back.Visible:=False;
      Title.Visible := False;
      Legend.Visible := true;
      Legend.Transparent:=false;
      View3D := False;
      Panning.MouseWheel:=pmwNone;
      Legend.ResizeChart:=false;
      Legend.LegendStyle:=lsSeries;
      Axes.FastCalc := True;
      LeftAxis.Axis.Width:=1;
      BottomAxis.Axis.Width:=1;
      LeftAxis.Grid.Visible:=true;
      BottomAxis.Grid.Visible:=true;
      LeftAxis.RoundFirstLabel:=True;
      BottomAxis.RoundFirstLabel:=True;
      LeftAxis.LabelStyle:=talValue;
      BottomAxis.LabelStyle:=talValue;
      LeftAxis.AxisValuesFormat:='#,##0.#######';
      //BottomAxis.AxisValuesFormat:='#,##0.#######';
      LeftAxis.MaximumOffset:=20;
      Axes.Reset;
      LegendScrollbar.Horizontal:=false;
      LegendScrollbar.Bevel:=bvNone;
      LegendScrollbar.DrawStyle:=dsWhenNeeded;
    end;
    UpdateMemoryUse;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  UpdateMemoryUse;
end;

procedure TForm1.UpdateMemoryUse;
var j: Integer;
    nMarks: Integer;
begin
  Label1.Caption:=FormatFloat('memory use: ,'+FormatSettings.DecimalSeparator+'# K', CurrentMemoryUsage / 1024);

  nMarks:=0;
  for j:=0 to SpectraChart.SeriesCount-1 do
    Inc(nMarks,SpectraChart.Series[j].Marks.Items.Count);

  Label1.Caption:=Label1.Caption+', number of series: '+IntToStr(SpectraChart.SeriesCount)+
                  ', number of marks: ' + FormatFloat(','+FormatSettings.DecimalSeparator+'#', visibleMarks);
end;

procedure TForm1.SeriesAfterDrawValues(Sender: TObject);
var i, j: Integer;
begin
  if Sender is TFastLineSeries then
  with TFastLineSeries(Sender) do
  begin
    for i:=0 to seriesMarks.Count-1 do
      for j:=0 to seriesMarks[i].Count-1 do
        with seriesMarks[i][j], Mark do
        begin
          if (Position=nil) or (not Position.Custom) then
          begin
            ShapeBounds.Left:=CalcXPos(ValueIndex);
            ShapeBounds.Top:=CalcYPos(ValueIndex) - Marks.ArrowLength - Marks.Callout.Distance;
            TCustomTextShapeAccess(Mark).CalcBounds(SpectraChart);

            ShapeBounds.SetLocation(ShapeBounds.Left - (ShapeBounds.Width div 2), ShapeBounds.Top - ShapeBounds.Height);
          end
          else
          begin
            ShapeBounds.Left:=Position.LeftTop.X;
            ShapeBounds.Top:=Position.LeftTop.Y;
            TCustomTextShapeAccess(Mark).CalcBounds(SpectraChart);
          end;

          seriesMarks[i][j].Mark.Draw;
        end;
  end;
end;

procedure TForm1.SpectraChartMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var i, j: Integer;
begin
  for i:=0 to seriesMarks.Count-1 do
    for j:=0 to seriesMarks[i].Count-1 do
      with TCustomTextShapeAccess(seriesMarks[i][j].Mark) do
        if ClickedShape(Point(X,Y),ShapeBounds) then
        begin
          dragMarksTool.SeriesIndex:=i;
          dragMarksTool.ValueIndex:=j;
          dragMarksTool.IOldPos:=Point(X,Y);
          Break;
        end;
end;

procedure TForm1.SpectraChartMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var i, j: Integer;
    DifX : Integer;
    DifY : Integer;
begin
  if (dragMarksTool.SeriesIndex>=0) and (dragMarksTool.ValueIndex>=0) then
  begin
    with seriesMarks[dragMarksTool.SeriesIndex][dragMarksTool.ValueIndex] do
    begin
      DifX:=X-dragMarksTool.IOldPos.X;
      DifY:=Y-dragMarksTool.IOldPos.Y;

      if Position=nil then
      begin
        Position:=TSeriesMarkPosition.Create;
        Position.Offset(Point(Mark.ShapeBounds.Left,Mark.ShapeBounds.Top));
      end;

      Position.Custom:=True;
      Position.Offset(Point(DifX,DifY));
      dragMarksTool.IOldPos:=Point(X,Y);
      SpectraChart.CancelMouse:=True;
      SpectraChart.Repaint;
    end;

    SpectraChart.Zoom.Active:=False;
    SpectraChart.CancelMouse:=True;
  end
  else
    for i:=0 to seriesMarks.Count-1 do
      for j:=0 to seriesMarks[i].Count-1 do
        with TCustomTextShapeAccess(seriesMarks[i][j].Mark) do
          if ClickedShape(Point(X,Y),ShapeBounds) then
          begin
            ParentChart.Cursor:=crHandPoint;
            ParentChart.CancelMouse:=True;
            Break;
          end;
end;

procedure TForm1.SpectraChartMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (dragMarksTool.SeriesIndex>=0) and (dragMarksTool.ValueIndex>=0) then
  begin
    dragMarksTool.SeriesIndex:=-1;
    dragMarksTool.ValueIndex:=-1;
    SpectraChart.CancelMouse:=True;
  end;
end;

end.



